/*  Copyright (C) 2001    Dobin Rutishauser        dobin.rutishauser@gmx.net
                          Bjrn Paetzel                       kolrabi@gmx.de
   
  ------------------------------------------------------------------------

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation;  either  version 2 of the  License, or
    any later version.

    This program is distributed in the hope that it  will be useful, but
    WITHOUT  ANY  WARRANTY;  without   even  the  implied   warranty  of
	MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE.  See the  GNU
    General Public License for more details.
	
	You should have received a copy  of the  GNU  General Public License
    along with this program (GPL.TXT); if not, write to the 

	Free Software Foundation, Inc., 
	59 Temple Place, Suite 330, 
	Boston, MA
    02111-1307, USA.
*/

// ---------------------------------------------------------- //
// scan.c

// ---------------------------------------------------------- //
// includes

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>


#include <libnet.h>
#include <pcap.h>


#include "../common/messages.h"
#include "../common/config.h"
#include "../common/utils.h"

#include "serversockets.h"
#include "serverconfig.h"
#include "scan.h"


/* pcap while alarm timeout flag */
unsigned int pcap_flag = 0;

/* alarm handler
 *
 * set timeout flag for pcap
 *
*/

int alarm_handler(int signo)
{
	pcap_flag = 0;
	return(0);
}


/* recv packet
 *
 * needs an destination ip and port, scan mode and timeout
 *
 * wait's for acknowledgement, sended by dst_ip from port dst_prt
*/

int recvpacket(int dst_ip, u_short src_prt, int mode, int timeout, u_long seq_nr)
{
struct pcap_pkthdr usefull;
u_char *packet;
struct ip *ippkt;
struct tcphdr *tcppkt;

	struct ether_header {
        u_char ether_dhost[6]; /* destination host */
        u_char ether_shost[6]; /* source host */
        u_short ether_type;    /* frame type */
	} *ethpkt;

	/* setitimer is too complicated */
	alarm(timeout);
	pcap_flag = 1;

   while (pcap_flag == 1)
   {
      packet = (u_char *) pcap_next(libpcap_socket, &usefull);
      if (packet != NULL)
      {
			
			ethpkt = (struct ether_header *)(packet);
			if (ntohs (ethpkt->ether_type) != ETHERTYPE_IP) continue;
		
			ippkt = (struct ip*)(packet + LIBNET_ETH_H);
			if (ippkt->ip_p != 6) continue;
			
         tcppkt = (struct tcphdr*)(packet + LIBNET_ETH_H + LIBNET_IP_H);

			if (dst_ip == ippkt->ip_src.s_addr && src_prt == ntohs(tcppkt->th_sport) && seq_nr+1 == ntohl(tcppkt->th_ack))
			{
				if (mode == 1) {   /* SYN SCAN */

					if (tcppkt->th_flags & TH_SYN && tcppkt->th_flags & TH_ACK)
					{
						return(1);
					}

					if (tcppkt->th_flags & TH_RST) {
						return(0);
					}
				} 
				else
				{  /* NULL FIN XMAS ACK SCAN */

					if (tcppkt->th_flags & TH_RST) {
						return(0);
					} 
					else
					{
						return(1);
					}
				}
			}
      }
   }

	/* timeout */
	if (mode == 1) {
		return(0);
	} else {
		return(1);
	}
}


/* send packet
 *
 * needs a destination ip and port, the libnet socket and a mode
 *
 * sends a packet to dst_ip and dst_prt througth libnet_socket, in mode
 * mode (1 is for activating syn etc..)
 *
*/

int sendpacket(u_long dst_ip, u_short dst_prt, int mode, u_short src_port, u_long seq_nr)
{
	int packet_size, c;
	char localhostname[128];
	u_char *packet;
	u_long src_ip;
	u_char flags;

	gethostname(localhostname, sizeof(localhostname));

	if (!(src_ip = libnet_name_resolve(localhostname, LIBNET_RESOLVE))) 
	{ 
		libnet_error(LIBNET_ERR_FATAL, "Bad source IP address: %s\n", optarg);
		return(0); 
	} 

	packet_size = LIBNET_IP_H + LIBNET_TCP_H; 

	libnet_init_packet(packet_size, &packet); 
	if (packet == NULL) 
	{ 
		libnet_error(LIBNET_ERR_FATAL, "libnet_init_packet failed\n"); 
	} 

	/* build IP Header */
	libnet_build_ip(LIBNET_TCP_H, IPTOS_LOWDELAY, 242, 0, 48, IPPROTO_TCP,
		src_ip, dst_ip, NULL, 0, packet);

	/* build TCP Header */
	switch(mode)
	{
		case 0:
			flags = NULL;
			break;

		case 1:
			flags = TH_SYN;
			break;

		case 2:
			flags = TH_FIN;
			break;

		case 3:
			flags = TH_FIN | TH_PUSH | TH_URG;
			break;

		case 4:
			flags = TH_ACK;
			break;
	}

	libnet_build_tcp(src_port, dst_prt, seq_nr, (u_long)(rand()), flags, 
		(u_short)(rand()), 0, NULL, 0, packet + LIBNET_IP_H);

	if (libnet_do_checksum(packet, IPPROTO_TCP, LIBNET_TCP_H) == -1) 
	{ 
		libnet_error(LIBNET_ERR_FATAL, "libnet_do_checksum failed\n"); 
	} 

	c = libnet_write_ip(libnet_socket, packet, packet_size); 
	if (c < packet_size) { 
		libnet_error(LN_ERR_WARNING, "libnet_write_ip only wrote %d bytes\n", c); 
		libnet_destroy_packet(&packet); 
		return(0);
	} 
  
    libnet_destroy_packet(&packet); 

return (1); 
} 



/* stealth scan
 *
 * stealthscan handler
 *
*/
int stealthscan (char *server, unsigned short port, unsigned int mode, int speed) 
{
int res=0, dst_ip;
u_short src_port = (u_short)(rand());  /* create source port and seq */
u_long seq_nr = (u_long)(rand());      /* number for identify our packets */


	if (! (dst_ip = libnet_name_resolve(server, LIBNET_RESOLVE)))
	{
		libnet_error (LIBNET_ERR_FATAL, "Bad destination IP address: %s\n", server);
	}

	/* send the packet, bzw. scan the port */
   res = sendpacket(dst_ip, port, mode, src_port, seq_nr);
	if (res < 1) {
		logprintf (LVL_WARNING, "could not send packet" );
		return (PORT_UNKNOWN);
	}

	/* receive the answer */
   res = recvpacket(dst_ip, port, mode, speed, seq_nr);
   if (res == 1) {
		return(PORT_OPEN);
   } else {
		return(PORT_CLOSED);
   }
}


/*
 * connect scan
 *
 * connects to name and port
 * return PORT_OPEN or PORT_CLOSED
 *
*/
int connectscan (char *name, int port)
{
	SOCKET			scansocket;
	SOCKADDR_IN		saddress;
	struct			hostent *he;


	if ((he=gethostbyname(name)) == NULL) {  /* get the host info */
		perror("gethostbyname");
		exit(1);
	}

	if ((scansocket = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
		perror("socket");
		exit(1);
	}

	// set address and port
	saddress.sin_family			= AF_INET;
	saddress.sin_port				= htons( port );
	saddress.sin_addr.s_addr	= inet_addr( name );
	bzero(&(saddress.sin_zero), 8);

	if (connect (scansocket, (struct sockaddr *)&saddress, 
		sizeof(struct sockaddr)) == -1)
	{
		shutdown (scansocket,0);
		return (PORT_CLOSED);
	} else {
		shutdown (scansocket,0);
		return (PORT_OPEN);
	}

}


// -----                                                                   -- //
// scan()                                                           21. 04. 2001
//
// parameters:
//
// char  	*name		name of the server to scan
//				this must be a dotted number string
// int		port		port number to scan
// int		type		type of scan to do
// int		timeout		port timeout
//
// return values
//
// PORT_OPEN or PORT_CLOSED or PORT_UNKNOWN
//
// scans a host if a given port is open
// --                                                                   ----- //

int		scan( char *name, int port, int type, int timeout )
{
	signal(SIGALRM, alarm_handler);

	printf("."); fflush(stdout);

	switch(type)
	{
		case	SCAN_CONNECT:
			return connectscan( name, port );
			break;

		case	SCAN_SYN:
			return stealthscan(name, port, 1, timeout);
			break;

		case	SCAN_FIN:
			return stealthscan(name, port, 2, timeout);
			break;
			
		case	SCAN_XMAS:
			return stealthscan(name, port, 3, timeout);
			break;
			
		case	SCAN_NULL:
			return stealthscan(name, port, 0, timeout);
			break;
			
		case	SCAN_ACK:
			return stealthscan(name, port, 4, timeout);
			break;
	}
	
	return PORT_CLOSED;
}
